@@ -8,6 +8,7 @@ import logging
 import datetime
 from extract import __version__
 from optparse import OptionParser
+import traceback as tb
 
 log = logging.getLogger("geonode-extract")
 
@@ -127,6 +128,24 @@ def download_layer(layer, url,  dest_dir, username=None, password=None):
             layer_file.write(content)
             log.debug('Saved data from "%s" as "%s"' % (layer['name'], layer_filename))
 
+
+    base_filename, extension = os.path.splitext(layer_filename)
+ 
+    # If this file a zipfile, unpack all files with the same base_filename
+    # and remove the downloaded zip
+    if zipfile.is_zipfile(layer_filename):
+        log.debug('Layer "%s" is zipped, unpacking now' % layer_filename)
+        # Create a ZipFile object
+        z = zipfile.ZipFile(layer_filename)
+        for f in z.namelist():
+            log.debug('Found "%s" in "%s"' % (f, layer_filename)) 
+            _, extension = os.path.splitext(f)
+            filename = base_filename + extension
+            log.debug('Saving "%s" to "%s"' % (f, filename))
+            z.extract(f, filename)
+        log.debug('Removing "%s" because it is not needed anymore' % layer_filename)
+        os.remove(layer_filename)
+
     # metadata_links is originally a list of lists, each item looks like:
     # ['text/xml', 'TC211', 'http://...//'], this operation
     # transforms it into a simple dict, with items like:
@@ -134,7 +153,6 @@ def download_layer(layer, url,  dest_dir, username=None, password=None):
     metadata_links = dict([ (b, c) for a, b, c in layer['metadata_links']])
     metadata_link = metadata_links['TC211']
 
-    base_filename, extension = os.path.splitext(layer_filename)
     metadata_filename = base_filename + '.xml'
     try:
         # Download the file
@@ -216,18 +234,26 @@ def get_data(argv=None):
     log.info('Processing %s layers' % number)
     output = []
     for i, layer in enumerate(layers):
-        try:
-            download_layer(layer, url, dest_dir, username, password)
-        except Exception, e:
-            log.exception('Could not download layer "%s".' % layer['name']) 
-            exception_type, error, traceback = sys.exc_info()
-            status = 'failed'
-            if not ignore_errors:
-                msg = "Stopping process because --ignore-errors was not set and an error was found."
-                log.error(msg)
-                sys.exit(-1)
+        if ':' in layer['name']:
+            name = layer['name'].split(':')[1]
         else:
-            status = 'downloaded'
+            name = layer['name']
+
+        if os.path.exists(os.path.join(dest_dir, name + '.sld')):
+            status = 'skipped'
+        else:
+            try:
+                download_layer(layer, url, dest_dir, username, password)
+            except Exception, e:
+                log.exception('Could not download layer "%s".' % layer['name']) 
+                exception_type, error, traceback = sys.exc_info()
+                status = 'failed'
+                if not ignore_errors:
+                    msg = "Stopping process because --ignore-errors was not set and an error was found."
+                    log.error(msg)
+                    sys.exit(-1)
+            else:
+               status = 'downloaded'
 
         info = {'name': layer['name'], 'title': layer['title'], 'status': status}
         msg = "[%s] Layer %s (%d/%d)" % (info['status'], info['name'], i+1, number)
@@ -250,8 +276,8 @@ def get_data(argv=None):
     log.debug("\nDetailed report of failures:")
     for dict_ in output:
         if dict_['status'] == 'failed':
-            log.debug("\n\n", dict_['name'], "\n================")
-            traceback.print_exception(dict_['exception_type'],
+            log.debug(dict_['name'])
+            tb.print_exception(dict_['exception_type'],
                                       dict_['error'],
                                       dict_['traceback'])
 
